##############################################################
#
#   Humerus - Example game, with levels
#
#   An extended version of the pong example with levels.
#
#   A level consist of a text file containing one
#   integer representing the size of the bat.
#
##############################################################

#
#  This is so you can run the example from the source
#  directory without installing humerus.
#

import os, sys
d = os.path.dirname
sys.path.insert(1, d(d(d(os.path.abspath(sys.argv[0])))))

#
#  Parse command line options and initialize display
#  before doing anything else.
#

from humerus.options import Options

class PongOptions(Options):

	frame_rate_option = 20
	screen_size_option = (640, 480)

options = PongOptions()

#
#  Now do the rest.
#

import random
from pygame import Rect
from albow.screen import Screen
from albow.controls import Label
from humerus.hlgame import HLGame
from humerus.hlshell import HLShell
from humerus.main_commands import MainCommandScreen

class Level(object):
	
	def __init__(self, bat_size):
		self.bat_size = bat_size
		self.score = 0
		self.ball_position = None
		self.is_completed = False
	
	def level_is_completed(self):
		result = self.score >= 3
		return result
	
	def begin_frame(self):
		p = self.ball_position
		if p:
			x, y = p
			vx, vy = self.ball_speed
			w, h = shell.size
			x += vx
			y += vy
			if x < 0:
				x = -x
				vx = -vx
			elif x > w:
				r = game.bat_rect
				if r and not (y >= r.top and y <= r.bottom):
					self.ball_position = None
					self.score -= 1
					return
				else:
					self.score += 1
					x = w - (x - w)
					vx = -vx
			if y < 0:
				y = -y
				vy = -vy
			elif y > h:
				y = h - (y - h)
				vy = -vy
			self.ball_position = x, y
			self.ball_speed = vx, vy

class PongGame(HLGame):

	save_file_suffix = ".lpongam"
	level_file_suffix = ".lponglev"
	level_set_suffix = ".lpongset"

	bat_rect = None
	
	def read_level(self, path):
		f = open(path)
		bat_size = int(f.readline())
		f.close()
		return Level(bat_size)

help_text = \
"""Click the mouse to serve.\n
Hit the ball to gain a point.\n
You lose a point when you miss.\n
Get 3 points to advance to the next level."""

class PongHelpScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		help = Label(help_text)
		self.add_centered(help)

class PongPlayScreen(MainCommandScreen):

	ball_color = (255, 255, 255)
	ball_size = (8, 8)
	bat_color = (255, 255, 255)
	bat_width = 8

	def draw(self, surf):
		level = game.playing_level
		r = self.ball_rect()
		if r:
			surf.fill(self.ball_color, r)
		r = game.bat_rect
		if r:
			surf.fill(self.bat_color, r)
		name = game.level_name.split(".")[0]
		s = "Level: %s     Score: %s" % (name, level.score)
		buf = self.font.render(s, True, self.fg_color)
		r = buf.get_rect()
		r.midbottom = self.width // 2, self.height
		surf.blit(buf, r)
	
	def ball_rect(self):
		p = game.playing_level.ball_position
		if p:
			r = Rect((0, 0), self.ball_size)
			r.center = p
			return r
	
	def begin_frame(self):
		e = self.get_mouse()
		x, y = e.local
		r = Rect(0, 0, self.bat_width, game.level.bat_size)
		r.midright = self.width, y
		game.bat_rect = r
		self.invalidate()
	
	def mouse_down(self, e):
		level = game.playing_level
		if level.ball_position is None:
			r = game.bat_rect
			if r:
				level.ball_position = r.midleft
				vx = -10
				vy = random.randint(-10, 10)
				level.ball_speed = vx, vy
		
class PongShell(HLShell):

	game_title = "Python Pong"

	def __init__(self, game, options):
		HLShell.__init__(self, game, options)
		self.about_screen = PongHelpScreen(self)
		self.main_menu_screen = MainCommandScreen(self, with_menu = True)
		self.play_screen = PongPlayScreen(self)


game = PongGame()
shell = PongShell(game, options)
shell.run()
